@@ -2,7 +2,12 @@ import os
 import tempfile
 from unittest import TestCase
 
-from pipeline import Message, Generator, Processor, Splitter
+from pipeline import (
+    Message,
+    GeneratorConfig, Generator,
+    ProcessorConfig, Processor,
+    SplitterConfig, Splitter,
+)
 
 
 class TestWorkerCore(TestCase):
@@ -34,7 +39,8 @@ class TestWorkerCore(TestCase):
                 for i in range(3):
                     self.dataWriter.write(i, dict([(field, field) for field in self.dataWriter.fields]))
                     yield {"key": i}
-        generator = MyGenerator('generator', '0.1.0', dataKind='MEM')
+        config = GeneratorConfig(dataKind='MEM')
+        generator = MyGenerator('generator', '0.1.0', config=config)
         generator.parse_args(args=[
             '--kind', 'MEM', '--out-topic', 'test',
             '--out-fields', 'key1,key2']
@@ -46,19 +52,22 @@ class TestWorkerCore(TestCase):
         assert generator.dataWriter.results[0]['key1'] == 'key1'
 
     def test_generator_invalid_message(self):
+        valids = [True, True, False]
+
         class InvalidMessage(Message):
             def is_valid(self):
-                return False
+                return self.dct.get('valid', False)
 
         class MyGenerator(Generator):
             def generate(self):
-                for i in range(3):
-                    yield {"key": i}
+                for i, v in enumerate(valids):
+                    yield {"key": i, "valid": v}
 
-        generator = MyGenerator('generator', '0.1.0', messageClass=InvalidMessage)
+        config = GeneratorConfig(messageClass=InvalidMessage)
+        generator = MyGenerator('generator', '0.1.0', config=config)
         generator.parse_args(args=['--kind', 'MEM', '--out-topic', 'test'])
         generator.start()
-        assert len(generator.destination.results) == 0
+        assert len(generator.destination.results) == 2
 
     def test_processor_invalid_message(self):
         class InvalidMessage(Message):
@@ -66,7 +75,8 @@ class TestWorkerCore(TestCase):
                 return False
 
         msgs = [{}, {}, {}]
-        pro1 = Processor('tester1', '0.1.0', messageClass=InvalidMessage)
+        config = ProcessorConfig(messageClass=InvalidMessage)
+        pro1 = Processor('tester1', '0.1.0', config=config)
         pro1.parse_args(args=['--kind', 'MEM', '--out-topic', 'test'], config={'data': msgs})
         pro1.use_retry_topic('optional-retry-topic')
         pro1.start()
@@ -75,7 +85,7 @@ class TestWorkerCore(TestCase):
 
     def test_processor_retry(self):
         class RetryProcessor(Processor):
-            def process(self, dct):
+            def process(self, msg):
                 return "Error"
         msgs = [{'key': '1'}, {'key': '2'}, {'key': '3'}]
         pro1 = RetryProcessor('tester1', '0.1.0')
@@ -85,10 +95,24 @@ class TestWorkerCore(TestCase):
         assert len(pro1.destination.results) == 0
         assert len(pro1.retryDestination.results) == 3
 
+    def test_processor_terminated_messages(self):
+        class RetryProcessor(Processor):
+            def process(self, msg):
+                msg.terminates()
+                return None
+        msgs = [{'key': '1'}, {'key': '2'}, {'key': '3'}]
+        pro1 = RetryProcessor('tester1', '0.1.0')
+        pro1.parse_args(args=['--kind', 'MEM', '--out-topic', 'test'], config={'data': msgs})
+        pro1.use_retry_topic('optional-retry-topic')
+        pro1.start()
+        assert len(pro1.destination.results) == 0
+        assert len(pro1.retryDestination.results) == 0
+
     def test_processor(self):
         class MyProcessor(Processor):
-            def process(self, dct):
-                dct['newkey'] = 'newval'
+            def process(self, msg):
+                updates = {'newkey': 'newval'}
+                msg.update(updates)
                 return None
         msgs = [{'key': '1'}, {'key': '2'}, {'key': '3'}]
         pro1 = MyProcessor('tester1', '0.1.0')
@@ -96,15 +120,20 @@ class TestWorkerCore(TestCase):
         pro1.start()
         assert len(pro1.destination.results) == 3
         m = pro1.destination.results[0]
-        assert m.dct['newkey'] == 'newval'
+        assert m.get('newkey') == 'newval'
 
     def test_processor_with_data(self):
         class MyProcessor(Processor):
-            def process(self, dct):
-                self.dataWriter.write(dct['key'], {'key3': self.dataReader.read(dct['key'])})
+            def process(self, msg):
+                self.logger.info("%s, %s", msg.dct.get('key', None), msg.get('key'))
+                self.dataWriter.write(
+                    msg.get('key'),
+                    {'key3': self.dataReader.read(msg.get('key'))}
+                )
                 return None
         msgs = [{'key': 'm1'}, {'key': 'm2'}, {'key': 'm3'}]
-        pro1 = MyProcessor('tester1', '0.1.0', dataKind='MEM')
+        config = ProcessorConfig(dataKind='MEM')
+        pro1 = MyProcessor('tester1', '0.1.0', config=config)
         pro1.parse_args(
             args='--kind MEM --out-topic test --in-fields key1,key2 --out-fields key3'.split(),
             config={
@@ -119,8 +148,8 @@ class TestWorkerCore(TestCase):
         pro1.start()
         assert len(pro1.destination.results) == 3
         m = pro1.dataWriter.results['m1']
-        assert m['key3']['key1'] == 'val11'
-        assert m['key3']['key2'] == 'val21'
+        assert m.get('key3')['key1'] == 'val11'
+        assert m.get('key3')['key2'] == 'val21'
 
     def test_file(self):
         processor = Processor('fileprocessor', '0.1.0')
@@ -156,7 +185,8 @@ class TestWorkerCore(TestCase):
         assert len(pro1.destination.results) == 3
 
     def test_mem_processor_nooutput(self):
-        pro1 = Processor('tester2', '0.1.0', nooutput=True)
+        config = ProcessorConfig(noOutput=True)
+        pro1 = Processor('tester2', '0.1.0', config=config)
         pro1.parse_args(args=['--kind', 'MEM'], config={'data': [{}]})
         pro1.start()
         assert not hasattr(pro1, 'destination')
@@ -177,7 +207,8 @@ class TestWorkerCore(TestCase):
                 return False
 
         msgs = [{"key": 1, 'language': 'en'}, {"key": 2, 'language': 'it'}]
-        splitter = Splitter('spliter1', '0.1.0', messageClass=InvalidMessage)
+        config = SplitterConfig(messageClass=InvalidMessage)
+        splitter = Splitter('spliter1', '0.1.0', config=config)
         splitter.parse_args(args=['--kind', 'MEM', '--out-topic', 'test'], config={'data': msgs})
         splitter.start()
         assert len(splitter.destinations['test-en'].results) == 0
@@ -195,7 +226,8 @@ class TestWorkerCore(TestCase):
         class CustomMessage(Message):
             def __str__(self):
                 return 'Message({}: {})'.format(self.dct['key'], self.dct['value'])
-        pro1 = Processor('tester3', '0.1.0', messageClass=CustomMessage)
+        config = ProcessorConfig(messageClass=CustomMessage)
+        pro1 = Processor('tester3', '0.1.0', config=config)
         pro1.parse_args(args=['--kind', 'MEM'], config={'data': [{'key': 'key1', 'value': 'value1'}]})
         pro1.start()
-        assert pro1.destination.results[0].dct['key'] == 'key1'
+        assert pro1.destination.results[0].get('key') == 'key1'
